const config = require('config')
const last = require('lodash/last')

const statuses = config.get('statuses')

const addHandlingEditor = async (collection, user, invitation) => {
  collection.handlingEditor = {
    id: user.id,
    name: `${user.firstName} ${user.lastName}`,
    timestamp: invitation.timestamp,
    email: user.email,
    hasAnswer: invitation.hasAnswer,
    isAccepted: invitation.isAccepted,
  }
  await updateStatus(collection, 'heInvited')
}

const updateHandlingEditor = async (collection, isAccepted) => {
  collection.handlingEditor.hasAnswer = true
  collection.handlingEditor.isAccepted = isAccepted
  collection.handlingEditor.timestamp = Date.now()
  let status
  isAccepted ? (status = 'heAssigned') : (status = 'submitted')
  await updateStatus(collection, status)
}

const getFragmentAndAuthorData = async ({
  UserModel,
  FragmentModel,
  collection: { fragments, authors },
}) => {
  const fragment = await FragmentModel.find(last(fragments))
  let { title, abstract } = fragment.metadata
  title = title.replace(/<(.|\n)*?>/g, '')
  abstract = abstract ? abstract.replace(/<(.|\n)*?>/g, '') : ''

  const submittingAuthorData = authors.find(
    author => author.isSubmitting === true,
  )
  const author = await UserModel.find(submittingAuthorData.userId)
  const authorName = `${author.firstName} ${author.lastName}`
  const authorsPromises = authors.map(async author => {
    const user = await UserModel.find(author.userId)
    return `${user.firstName} ${user.lastName}`
  })
  const authorsList = await Promise.all(authorsPromises)
  const { id } = fragment
  return { title, authorName, id, abstract, authorsList }
}

const updateReviewerCollectionStatus = async collection => {
  const reviewerInvitations = collection.invitations.filter(
    inv => inv.role === 'reviewer',
  )
  if (reviewerInvitations.length === 0) return
  await updateStatus(collection, 'heAssigned')
}

const updateStatus = async (collection, newStatus) => {
  collection.status = newStatus
  collection.visibleStatus = statuses[collection.status].private
  await collection.save()
}

module.exports = {
  addHandlingEditor,
  updateHandlingEditor,
  getFragmentAndAuthorData,
  updateReviewerCollectionStatus,
  updateStatus,
}
